/* リスト4.9 Shadowputのプログラム */

import SimpleOpenNI.*;       // Kinectライブラリ（OpenNI）
import processing.serial.*;  // シリアル通信ライブラリ
import picking.*;            // ピッキングライブラリ
import cc.arduino.*;         // Arduinoライブラリ

SimpleOpenNI context; // OpenNIの処理をするオブジェクト
Arduino      arduino; // Arduinoとの通信を行うオブジェクト
Picker       picker;  // ピッキングを行うオブジェクト

int[]     port  = { 2, 3 }; // ポート番号
PVector[] pos   = { new PVector(100, 0), new PVector(200,0) }; // ボタンの位置
boolean[] state = { false, false };      // ボタンの状態
boolean[] prev_touch = { false, false }; // 前フレームでのボタンへの接触状態
PImage    button_on, button_off;         // ボタンの画像
int       movable_id = -1;               // マウスで移動中のボタンのID

void setup() {
  // Presentモードにおける背景色の設定
  frame.setBackground(new java.awt.Color(255,255,255));
  
  // Kinectの設定
  context = new SimpleOpenNI(this);                   // SimpleOpenNIの準備
  context.enableDepth();                              // デプス画像有効
  context.setMirror(true);                            // 左右のミラー反転有効
  context.enableUser(SimpleOpenNI.SKEL_PROFILE_ALL);  // ユーザトラッキング有効

  // Arduinoの設定 （COMポートの番号は自分の環境に合わせてください）
  arduino = new Arduino(this, Arduino.list()[0], 57600);
  for (int i=0; i<port.length; i++) {
    arduino.pinMode( port[i], Arduino.OUTPUT ); // 入出力ポートを「出力」に設定
  }

  size(context.depthWidth(), context.depthHeight());  // 画面サイズの設定
  button_on = loadImage("button_on.png");             // ボタン画像（ON）のロード
  button_off = loadImage("button_off.png");           // ボタン画像（OFF）のロード
  picker = new Picker(this);                          // ピッキングの準備  
}

void draw() {
  background(255,255,255); // 背景の設定（背景色：白）
  context.update();        // Kinectの更新
  
  // ユーザマップの取得
  int[] userMap = null;
  int userCount = context.getNumberOfUsers();
  if (userCount > 0) {
    userMap = context.getUsersPixels(SimpleOpenNI.USERS_ALL);
  }

  // ユーザマップの情報を元に，ユーザのシルエットを描画
  loadPixels();
  for (int y=0; y<context.depthHeight(); y++) {
    for (int x=0; x<context.depthWidth(); x++) {
      int index = x + y * context.depthWidth();      
      if ( userMap != null && userMap[index] != 0 ) {
        pixels[index] = color(0,0,0); // シルエットの色：黒
      }
    } 
  }
  updatePixels();
  
  // ボタンの描画（ここでピッキング判定のためのID付与も行う）
  for (int id = 0; id < state.length; id++) {
    picker.start(id);
    draw_button(id);
  }
  picker.stop();

  // マウスによるボタンの移動
  if ( movable_id >= 0 ) {
    pos[movable_id].x = mouseX;
    pos[movable_id].y = mouseY;
  }
  
  // ユーザ毎の処理
  for (int userId=1; userId<=context.getNumberOfUsers(); userId++) {
    // 画面上での右手の位置を取得
    PVector realPos = new PVector();
    PVector screenPos = new PVector();  
    context.getJointPositionSkeleton( userId, SimpleOpenNI.SKEL_RIGHT_HAND, realPos );
    context.convertRealWorldToProjective( realPos, screenPos );
    
    // 右手の位置が画面内にあれば
    if ( screenPos.x>0 && screenPos.y<width && screenPos.y>0 && screenPos.y<height ) {
      // 右手の位置に円を描画
      fill(255,255,0);
      ellipse(screenPos.x, screenPos.y, 20, 20);
      
      // 各ボタンに対する右手の接触判定と家電のスイッチの切り替え
      for (int id = 0; id < state.length; id++) {
        boolean current_touch = (picker.get(int(screenPos.x), int(screenPos.y)) == id);
        if (current_touch && !prev_touch[id]) {   // 「接触なし」→「接触あり」となっ
          switch_state(id);                       // たら家電のOn/Offを行う
        }
        prev_touch[id] = current_touch;
      }
    }
  }
}

// マウスによるボタンのピッキング判定（ボタンを移動させるための処理）
void mousePressed() {
  movable_id = picker.get(mouseX,mouseY);  
}

// マウスボタンを離した際に，移動中のボタンのIDをリセットする
void mouseReleased() {
  movable_id = -1;
}

// ボタンの描画
void draw_button(int id) {
  rect(pos[id].x, pos[id].y, 50, 50); // ピッキング判定用の四角形
  image(state[id] ? button_on : button_off,  pos[id].x, pos[id].y, 50, 50);
}

// スイッチのOn/Off
void switch_state(int id) {
  state[id] = !state[id];
  int value = (state[id] ? Arduino.HIGH : Arduino.LOW);  
  arduino.digitalWrite(port[id], value);  
}

// 新しくユーザが見つかった際の処理
void onNewUser(int userId) {
  context.startPoseDetection("Psi",userId); // Ψポーズの検出開始
}

// ポーズ開始時の処理
void onStartPose(String pose,int userId) {
  context.stopPoseDetection(userId);                // ポーズ検出を停止
  context.requestCalibrationSkeleton(userId, true); // キャリブレーションを指示
}

// キャリブレーションが終了した際の処理
void onEndCalibration(int userId, boolean successfull) {
  if (successfull) { 
    context.startTrackingSkeleton(userId);  // 成功ならトラッキング開始
  } else {
    context.startPoseDetection("Psi",userId); // 失敗ならΨポーズの検出開始
  }
}
